#!/bin/ksh
# *****************************************************************************
#  Prepare the environment to run the Service Profiler.
# *****************************************************************************
#  Wrightia, S.A. (c) 2006-2011
#
#  Input parameters:
#     $1          : JVM_DIR (The JVM installation directory)
#     $2          : IS_DIR  (The Integration Server installation directory)
#    [$3]         : Optional...
#                   Java List Separator.
#                   Default is ":".
#    [$4]         : Optional...
#                   Contains VOID when the Core Libraries are being
#                   intentionally prevented from loading.
#                   The call is made on this situation only to log the
#                   occurence.
#
#  Results:
#    The script creates 3 (three) values to be used by the
#    caller:
#
#    ISSPROF_JVM - Options to be added to the JVM command line
#    ISSPROF_CP  - Path entries to the added to the -cp (CLASSPATH) of the
#                  JVM command line
#    ISSPROF_LIB - Path to the folder/directory where the needed dynamic
#                  libraried are installed.
#                  This must be addded to whatever setting that can be used
#                  by the runtime to locate and use the dynamic libraries.
#
#  Exit codes:
#     0  - OK
#     1  - Help display requested
#     2  - Invalid arguments
#     3  - Intentionally VOID
#     4  - JRE home directory not found
#     5  - JRE executable not found
#     6  - IS home directory not found
#     7  - Error generating ClassLoader
#
# *****************************************************************************


# *****************************************************************************
# Define functions
# *****************************************************************************
displayUsage() {
   ############################################################################
   # Display script usage imformation

   echo "\nWrightia Service Profiler (c) 2006-2011\n"
   echo "  Usage:"
   echo "      prepare.sh <java_dir> <is_dir> [<jls>]\n"
   echo "  where:"
   echo "      java_dir    : Java installation directory"
   echo "      is_dir      : Integration Server installation directory"
   echo "      jls         : Java List Separator"
   echo "                    Optional."
   echo "                    Default is \":\".\n"
}

initLog() {
   ############################################################################
   # Sets and initializes the log file.
   # The initialization message is also sent to stderr.

   xLogFilePath=$mToolHome/logs
   mkdir -p "$xLogFilePath"
   mLogFileName=$xLogFilePath/issprof.log
   echo "$1"
   echo "$1"                                                                        >  "$mLogFileName"
   echo "=========================================================================" >> "$mLogFileName"
   date                                                                             >> "$mLogFileName"
   echo "=========================================================================" >> "$mLogFileName"
   unset xLogFilePath
}

logMessage() {
   ############################################################################
   # Send the message passed as argument to the log file... and to the console.
   #     $1    - message

   echo  "$1"
   echo  "$1" >> $mLogFileName
}

validateArguments () {
   ############################################################################
   # Validate script input arguments:
   #     $1    - java_dir
   #     $2    - IS_dir

   if [ ! -d "$1" ]; then
      logMessage "ERROR: JRE home directory not found: $1"
      exit 4
   elif [ ! -x "$1/bin/java" ]; then
      logMessage "ERROR: JRE not found."
      exit 5
   elif [ ! -d "$2" ]; then
      logMessage "ERROR: IS home directory not found."
      exit 6
   fi
}

defineJvmOptions() {
   ############################################################################

   xJRE=$mJavaHome/bin/java

   # activate native library files
   `cd "${ISSPROF_LIB}"; chmod +x lib*`

   xInterceptorCP="${ISSPROF_LIB}/wia-profiler-intc-is-11.jar"
   xClassLoaderDir="$mToolHome/tmp"
   mkdir -p "$xClassLoaderDir"
   mClassLoaderFileName="${xClassLoaderDir}/issprof-cloader.jar"

   logMessage "INFO: Removing any previously generated ClassLoader..."
   rm -f "$mClassLoaderFileName"

   logMessage "INFO: Generating new ClassLoader..."
   "$xJRE" -Dwm.server.init="${mISHome}/bin/ini.cnf" -cp "$xInterceptorCP" com.wrightia.wm.is.profiler.intercept.IssprofJarBuilder "$mClassLoaderFileName"
   if [ "$?" != "0" ]; then
      logMessage "ERROR: Error generating ClassLoader ($mClassLoaderFileName)"
      exit 7
   fi
   logMessage "INFO: ClassLoader generated successfuly"

   # JVM Boot (-X...) arguments
   xJVM_XOptions="-Xbootclasspath/p:$mClassLoaderFileName${JLS}$xInterceptorCP"

   # Additional profiling properties can be set here
   xJVM_Options=
   ISSPROF_JVM="${xJVM_XOptions} ${xJVM_Options}"

   # Generating IS profile wrapper configuration
   xWrpConf_Filename="wrapper.conf"
   xWrpConf_Source="${mISHome}/../profiles/IS/configuration/${xWrpConf_Filename}"
   xWrpConf_Target="${mToolHome}/var/${xWrpConf_Filename}"

   if [ -f "$xWrpConf_Source" ] ; then
      xAlreadyConfigured=`grep "$xWrpConf_Target" "$xWrpConf_Source"`
      if [ -z "$xAlreadyConfigured" ] ; then
         # create service profiler wrapper include file
         logMessage "INFO: Generating IS profile wrapper configuration ($xWrpConf_Target)..."
         "$xJRE" -Dwm.server.init="${mISHome}/bin/ini.cnf" -cp "$xInterceptorCP" com.wrightia.wm.is.config.WrapperBuilder "$ISSPROF_LIB" "$ISSPROF_JVM" "$xWrpConf_Source" "$xWrpConf_Target"
         if [ "$?" != "0" ]; then
            logMessage "ERROR: Error generating IS profile wrapper configuration ($xWrpConf_Target)"
            exit 8
         fi
         logMessage "INFO: IS profile wrapper configuration file generated successfuly"

         # append service profiler configuration to IS profile wrapper configuration
         cp "$xWrpConf_Source" "${xWrpConf_Source}.beforeissprof"
         echo ""                                                                      >> "$xWrpConf_Source"
         echo "#********************************************************************" >> "$xWrpConf_Source"
         echo "# Wrightia Service Profiler configuration file                       " >> "$xWrpConf_Source"
         echo "#********************************************************************" >> "$xWrpConf_Source"
         echo "#include $xWrpConf_Target"                                             >> "$xWrpConf_Source"
      fi
   fi
   # Cleanup...
   unset xWrpConf_Filename
   unset xWrpConf_Source
   unset xWrpConf_Target

   # Cleanup...
   unset xJRE
   unset xInterceptorCP
   unset xJVM_XOptions
   unset xClassLoaderDir
}

logResults() {
   ############################################################################
   logMessage ""
   logMessage "Script Results"
   logMessage "=================================="
   logMessage "   ISSPROF_LIB = \"$ISSPROF_LIB\""
   logMessage "   ISSPROF_CP  = \"$ISSPROF_CP\""
   logMessage "   ISSPROF_JVM = \"$ISSPROF_JVM\""
   logMessage ""
}


# *****************************************************************************
# Main
# *****************************************************************************

if [ "$1" = "-h" ]; then
   displayUsage
   exit 1
elif [ "$1" = "-?" ]; then
   displayUsage
   exit 1
elif [ -z "$1" -o -z "$2" ]; then
   echo "ERROR: mandatory arguments are missing."
   displayUsage
   exit 2
fi

mJavaHome=$1
mISHome=$2

# maintain this evaluation for retro-compatibility
if [ "$3" = "-d64" ]; then
   mIs64Bit=Y
   shift
else
   mIs64Bit=N
fi

# set the java list separator to use
if [ -z "$3" ]; then
   JLS=":"
else
   if [ "$3" = "VOID" ]; then
      mIsVoid=Y
   else
      # because UNIX emulators for Windows (e.g., CygWin)
      # use Java with Windows rules... you should pass ";"
      JLS="$3"
      if [ "$4" = "VOID" ]; then
         mIsVoid=Y
      fi
   fi
fi

mToolHome=${mISHome}/serviceprofiler

# Define the LIB PATH
ISSPROF_LIB=${mToolHome}/lib

initLog "Preparing for Wrightia Service Profiler..."
if [ "$mIsVoid" = "Y" ]; then
   logMessage  "INFO: Core libraries intentionally prevented from loading"
   exit 3
else
   logMessage  "INFO: Java Directory = \"$mJavaHome\""

   validateArguments "${mJavaHome}" "${mISHome}"
   defineJvmOptions
   logResults

   # return the results
   print "RESULT: ISSPROF_LIB=\"$ISSPROF_LIB\"\tISSPROF_JVM='$ISSPROF_JVM'\tISSPROF_CP=\"$ISSPROF_CP\""

   exit 0
fi

